package com.jiayue.cppclient.job;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.watch.WatchMonitor;
import cn.hutool.core.io.watch.Watcher;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.jiayue.cppclient.util.Md5KeyForCppUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.File;
import java.nio.file.Path;
import java.nio.file.WatchEvent;
import java.util.HashMap;
import java.util.Map;

/**
 * 反向隔离目录下文件监控
 * @author jy
 * @since 2024/12/10
 */
@Slf4j
@Component
public class FxglFileMonitor {
    @Value("${client.fxglPath}")
    String fxglPath;
    @Value("${client.cloudAddress}")
    String cloudAddress;

    @PostConstruct
    public void fileCreateMonitor() {
        // 指定监听目录，实际应用需要由外部配置文件进行配置
        File file = FileUtil.file(fxglPath);
        if (ObjectUtil.isNull(file) && !file.exists()) {
            log.info("监听目录：{}，不存在，请检查并修改配置文件后重启程序。", fxglPath);
            return;
        }
        //监听所有事件
        WatchMonitor watchMonitor = WatchMonitor.create(file, WatchMonitor.EVENTS_ALL);
        watchMonitor.setWatcher(new Watcher(){
            @Override
            public void onCreate(WatchEvent<?> event, Path currentPath) {
//                Object obj = event.context();
//                log.info("创建："+currentPath+"->"+ obj );
            }

            @Override
            public void onModify(WatchEvent<?> event, Path currentPath) {
//                Object obj = event.context();
//                log.info("修改："+currentPath+"->"+ obj );
            }

            @Override
            public void onDelete(WatchEvent<?> event, Path currentPath) {
                new Thread(() -> {
                    String fileName = event.context().toString();
                    if (fileName.startsWith("NWP_") || fileName.startsWith("DQ_")){
                        // 回传文件状态
                        String[] fileNames = fileName.split("_");
                        String stationCode = fileNames[1];
                        try {
                            callBackFileDownloadStatusToCloud(stationCode,fileName,"in", "是");
                        } catch (Exception e) {
                            throw new RuntimeException(e);
                        }
                    }
                }).start();

//                log.info("删除："+currentPath+"->"+ fileName );
            }

            @Override
            public void onOverflow(WatchEvent<?> event, Path currentPath) {
                Object obj = event.context();
                log.info("Overflow："+currentPath+"->"+ obj );
            }
        });
        // 设置监听目录的最大深度，目录层级大于指定层级的变更将不再监听，不设置默认只监听当前目录
//        watchMonitor.setMaxDepth(3);
        // 启动监听
        watchMonitor.start();
    }

    /**
     * 回传云端文件状态
     *
     * @param stationCode
     * @param forecastFileName
     * @param type
     * @param status
     * @return
     */
    private void callBackFileDownloadStatusToCloud(String stationCode, String forecastFileName,String type,String status) throws Exception{
        String callbackSignStr = Md5KeyForCppUtil.callbackFileMd5Key(stationCode, forecastFileName,type,status);
        Map postParms = new HashMap<>(5);
        postParms.put("stationCode", stationCode);
        postParms.put("forecastFileName", forecastFileName);
        postParms.put("type", type);
        postParms.put("status", status);
        postParms.put("signStr", callbackSignStr);
        String body = HttpUtil.post(cloudAddress + "/updateCppFileStatus", postParms, 10000);
        log.info("反向隔离获取成功，回传云端文件已被隔离获取，返回结果："+body);
    }
}
